;*************************************************************************
;* 16C64/74 Parallel Slave port
;*	This program demonstrates the Parallel Slave Port function of 
;*	the PIC16C64/74.  The program is interrupt driven, when the PIC
;*	is either read from or written to, an interrupt is generated.  If the
;*	interrupt was caused by a read, a register is incremented, and 
;*	the new count is placed in an output queue.  If the interrupt was
;*	caused by a write, the data is put on the Port B pins
;
;       Program:          PSP64.ASM 
;       Revision Date:   
;                         1-15-97      Compatibility with MPASMWIN 1.40
;
;;**************************************************************************
	list p=16c64
        ERRORLEVEL  -302
;
	include "p16c64.inc"

;Register definitions
FLAGREG	equ	20h		;Flag bit register
OUTDATA	equ	21h		;Output data
INDATA	equ	22h		;Input data
COUNT	equ	23h		;Count of times output register read

;Bit definitions for flag register
err  	equ	00h		;Error flag bit
OUTRDY	equ	01h		;Output data ready flag
INFULL	equ	02h		;Input data received flag

	org	0000h		;Reset Vector
	goto	Start

	org	0005h		;Interrupt Vector
	goto	Service_Int

Start
	clrf	OUTDATA		;Clear data registers
	clrf	INDATA
	bsf	STATUS,RP0	;Select register page 1
	movlw	b'00010111'	;Set RD, WR, and CS as
	movwf	TRISE		; inputs, Enable Parallel Slave port
	movlw	0FFh
	movwf	TRISB		;Set Port_B to all outputs
	movlw	b'10000000'	;
	movwf	PIE1		;Enable Parallel Slave Port interrupt
	bcf	STATUS,RP0	;Select register page 0

	movf	OUTDATA,W	;Set output Data in PORTD
	movwf	PORTD		
	movlw	b'11000000'	;Set GIE, PEIE (enable interrupts)
	movwf	INTCON

Loop
	btfsc	FLAGREG,INFULL	;Check if input data received
	goto	Checkout	;No data ready, check output
	bcf	FLAGREG,INFULL	;Clear input data ready flag
	movf	INDATA,W	;Get Input data
	movwf	PORTB		;Output input data to Port_B
Checkout
	btfsc	FLAGREG,OUTRDY	;Check if data output already
	goto	Loop		;Not output yet, loop
	incf	COUNT, F	;Increment output data
	movf	COUNT,W		;Get output data
	movwf	OUTDATA		;Put data in output queue
	bsf	FLAGREG,OUTRDY	;Set flag for interrupt routine
	goto	Loop

;********************************************************************************
;*Interrupt Service Routine
;*	Inputs:	FLAGREG - Flag register to/from the main routine:
;*				Bit 1: OUTRDY -	To Service_Int, indicates data
;*						ready in output queue
;*		OUTDATA - Output data queue
;*		PIR1	- Interrupt flag register
;*		TRISE	- Parallel slave port flag register
;*		PORTD	- Input data from slave port
;*
;*	Outputs:
;*		PORTD	- Output data to slave port
;*		INDATA	- Input data queue
;*	       	FLAGREG - Flag register to/from the main routine:
;*				Bit 0: ERROR -  From Service_Int, indicates input 
;*						buffer overflow
;*				Bit 2: INFULL -	From Service_Int, indicates data
;*						received and in INDATA
;*********************************************************************************
 
Service_Int
	btfss	PIR1,PSPIF	;Test for Peripheral interrupt
	goto	Intout		;Not a Peripheral interrupt, exit
	bcf	PIR1,PSPIF	;Clear Peripheral interrupt
	bsf	STATUS,RP0	;Select Page 1
	btfss	TRISE,IBF	;Check if input data ready
	goto	Notinput	;No input, check output
	bcf	STATUS,RP0	;Input ready, select Page 0
	bsf	FLAGREG,INFULL	;Set flag for main routine
	movf	PORTD,W 	;Get input data
	movwf	INDATA		;Put byte in input queue
Notinput
	btfsc	TRISE,OBF	;Check if output data read
	goto	Intout		;Not read, exit
	bcf	STATUS,RP0	;Select Page 0
	btfss	FLAGREG,OUTRDY	;Check if data in output queue
	goto	Intout		;Output not read, exit
	movf	OUTDATA,W	;Get data from queue
	movf	PORTD, F	;Put data in output buffer
	bcf	FLAGREG,OUTRDY	;Clear flag for main routine
Intout
	bsf	STATUS,RP0	;Select Page 1
	btfsc	TRISE,IBOV	;Check input buffer overflow flag
	goto	Interror	;If not clear, error
	bcf	STATUS,RP0	;Select Page 0
	retfie			;Re-enable GIE and return
Interror
	bcf	STATUS,RP0	;Select Page 0
	bsf	FLAGREG,err	;Set error flag for main routine
	retfie			;Re-enable GIE and return
	
	end

